"""
w2lapp.schema.syntaxes: classes for known attribute types

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: syntaxes.py,v 1.70 2005/02/08 10:19:51 michael Exp $
"""

import re,types,imghdr,sndhdr,urllib,time,utctime,ldap,ldapurl,pyweblib.forms, \
       w2lapp.viewer,w2lapp.form,w2lapp.gui,mspki.util

from types import StringType,UnicodeType,IntType,LongType,FloatType

from w2lapp.core import utf2display

from ldaputil.base import is_dn

try:
  from cStringIO import StringIO
except ImportError:
  from StringIO import StringIO

try:
  from ldapoidreg import oid as oid_desc_reg
except ImportError:
  oid_desc_reg = {}


class SyntaxRegistry:

  def __init__(self):
    self.oid2syntax = ldap.cidict.cidict()
    self.at2syntax = ldap.cidict.cidict()

  def registerSyntaxClass(self,c):
    if type(c) is types.ClassType and hasattr(c,'oid'):
      self.oid2syntax[c.oid] = c

  def registerAttrType(self,syntax_oid,attrTypes):
    for a in attrTypes:
      self.at2syntax[a.strip()] = syntax_oid

  def syntaxClass(self,schema,attrtype_nameoroid):
    attrtype_oid = schema.getoid(ldap.schema.AttributeType,attrtype_nameoroid)
    if self.at2syntax.has_key(attrtype_oid):
      syntax_class = self.oid2syntax[self.at2syntax[attrtype_oid]]
    else:
      attrtype_se = schema.get_inheritedobj(ldap.schema.AttributeType,attrtype_oid,['syntax'])
      if attrtype_se is None or attrtype_se.syntax is None:
        syntax_class = LDAPSyntax
      else:
        syntax_class = self.oid2syntax.get(attrtype_se.syntax,LDAPSyntax)
    return syntax_class 


url_pattern  = r'^(ftp|http|https|news|snews|ldap|ldaps|mailto):(|//)[^ ]*'
url_regex  = re.compile(url_pattern)
labeleduri_regex = re.compile(url_pattern+r' .*')
timestamp_pattern = r'^([0-9]){12,14}(.[0-9]+)*Z$'
timestamp_regex  = re.compile(timestamp_pattern)
mail_pattern = r'^[\w@.=/_ -]+@[a-zA-Z0-9-]+(\.[a-zA-Z0-9-]+)*$'
mail_regex = re.compile(mail_pattern)
# See http://developer.novell.com/ndk/doc/ndslib/nds__enu/data/hniuyp90.html#hniuyp90


####################################################################
# Classes of known syntaxes
####################################################################

class LDAPSyntax:
  oid=''
  desc='Generic LDAP syntax'
  maxLen = w2lapp.cnf.misc.input_maxfieldlen
  maxValues = w2lapp.cnf.misc.input_maxattrs
  reObj=re.compile('^.*$')
  mimeType = 'application/octet-stream'
  fileExt = 'bin'
  editable = 1

  def __init__(self,attrType,attrValue=None,charset='utf-8'):
    self._charset = charset
    self.attrType = attrType
    self.attrValue = attrValue

  def sanitizeInput(self,attrValue):
    return attrValue

  def validate(self,attrValue):
    return self.reObj.match(attrValue)!=None

  def setValue(self,attrValue):
    self.attrValue = attrValue

  def getValue(self):
    return self.attrValue

  def ldapEncoding(self):
    if type(self.attrValue)==StringType:
      return self.attrValue
    elif type(self.attrValue)==UnicodeType:
      return self.attrValue.encode('utf-8')
    elif type(self.attrValue)==IntType or \
         type(self.attrValue)==LongType or \
         type(self.attrValue)==FloatType:
      return str(self.attrValue)
    else:
      raise ValueError,"self.attrValue has to be of type Integer, String or Unicode string"

  def formField(self):
    return pyweblib.forms.Input(
      self.attrType,
      ': '.join([self.attrType,self.desc]),
      self.maxLen,self.maxValues,self.reObj.pattern,default=self.attrValue,size=50
    )

  def getMimeType(self,form):
    return self.mimeType

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    if ldapurl.isLDAPUrl(self.attrValue):
      return LDAPUrl(self.attrType,self.attrValue).displayValue(sid,form,ls,dn,valueindex,commandbutton)
    rm = url_regex.search(self.attrValue)
    if rm!=None:
      rm = labeleduri_regex.match(self.attrValue)
      if rm!=None:
        url,label = self.attrValue.split(' ',1)
        return '<a href="%s/urlredirect?%s">%s</a>' % (form.script_name,url,utf2display(form.accept_charset,label))
      else:
        return '<a href="%s/urlredirect?%s">%s</a>' % (form.script_name,self.attrValue,self.attrValue)
    # Check for timestamp
    rm = timestamp_regex.match(self.attrValue)
    if rm!=None:
      return utctime.strftimeiso8601(utctime.strptime(self.attrValue))
    # No special format detected
    return utf2display(form.accept_charset,self.attrValue)


class Binary(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.5'
  desc = 'Binary'
  editable = 0

  def ldapEncoding(self):
    return self.attrValue

  def validate(self,attrValue):
    return 1

  def formField(self):
    f = pyweblib.forms.File(
      self.attrType,
      ': '.join([self.attrType,self.desc]),
      self.maxLen,self.maxValues,self.reObj.pattern,default=self.attrValue,size=50
    )
    f.mimeType = self.mimeType
    return f

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return form.applAnchor(
      'read','View/Load',sid,
      [('dn',dn),('read_attr',self.attrType),('read_attrindex',str(valueindex))]
    )


class Audio(Binary):
  oid = '1.3.6.1.4.1.1466.115.121.1.4'
  desc = 'Audio'
  mimeType = 'audio/basic'
  fileExt = 'au'

  def validate(self,attrValue):
    f = StringIO(attrValue)
    res = sndhdr.test_au(attrValue,f)
    return res!=None

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    mimetype = self.getMimeType(form)
    return """
      <embed
        type="%s"
        autostart="false"
        src="%s/read/%s?dn=%s&amp;read_attr=%s&amp;read_attrindex=%d"
      >
      %d bytes of audio data (%s)
      """ % (
        mimetype,
        form.script_name,sid,
        urllib.quote(utf2display(form.accept_charset,dn,escape_entities=0)),
        urllib.quote(self.attrType),
        valueindex,
        len(self.attrValue),
        mimetype
      )


class Certificate(Binary):
  oid = '1.3.6.1.4.1.1466.115.121.1.8'
  desc = 'Certificate Revocation List'
  mimeType = 'application/pkix-cert'
  fileExt = 'crt'

  def getMimeType(self,form):
    env = form.env
    if env.has_key('HTTP_USER_AGENT'):
      browsertype,browserversion = pyweblib.helper.BrowserType(
        env['HTTP_USER_AGENT']
      )
      if browsertype in ['Mozilla','Opera']:
        if self.attrType.lower().startswith('cacertificate'):
          return 'application/x-x509-ca-cert'
        else:
          return 'application/x-x509-email-cert'
    return self.mimeType

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return form.applAnchor(
      'read','View/Load',sid,
      [
        ('dn',dn),
        ('read_attr',self.attrType),
        ('read_attrindex',str(valueindex)),
        ('read_attrmode','view'),
      ]
    )


class CertificateRevocationList(Certificate):
  oid = '1.3.6.1.4.1.1466.115.121.1.9'
  desc = 'Certificate Revocation List'
  mimeType = 'application/pkix-crl'
  fileExt = 'crl'

  def getMimeType(self,form):
    env = form.env
    if env.has_key('HTTP_USER_AGENT'):
      browsertype,browserversion = pyweblib.helper.BrowserType(
        env['HTTP_USER_AGENT']
      )
      if browsertype in ['Mozilla','Opera']:
        return 'application/x-pkcs7-crl'
    return self.mimeType


class DirectoryString(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.15'
  desc = 'Directory String'
  charset = 'utf-8'

  def validate(self,attrValue):
    try:
      attrValue.encode(self.charset)
    except UnicodeError:
      return 0
    else:
      return 1

  def sanitizeInput(self,attrValue):
    if type(attrValue)==StringType:
      return unicode(attrValue,self.charset)
    elif type(attrValue)==UnicodeType:
      return attrValue
    else:
      raise ValueError, "Argument attrValue must be either StringType or UnicodeType"

  def getValue(self):
    return self.attrValue.encode(self.charset)

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return utf2display(form.accept_charset,self.attrValue)


class DistinguishedName(DirectoryString):
  oid = '1.3.6.1.4.1.1466.115.121.1.12'
  desc = 'Distinguished Name'

  def validate(self,attrValue):
    return is_dn(attrValue)

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return w2lapp.gui.DisplayDN(sid,form,ls,self.attrValue,commandbutton=commandbutton)

  def formField(self):
    return w2lapp.form.DistinguishedNameInput(
      self.attrType,': '.join([self.attrType,self.desc]),default=self.attrValue
    )


class NameAndOptionalUID(DistinguishedName):
  oid = '1.3.6.1.4.1.1466.115.121.1.34'
  desc = 'Name And Optional UID'

  def validate(self,attrValue):
    value = '#'.split(attrValue)
    return is_dn(value[0])

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    value = self.attrValue.split('#')
    dn_str = w2lapp.gui.DisplayDN(
      sid,form,ls,value[0],commandbutton=commandbutton
    )
    if len(value)==1 or not value[1]:
      return dn_str
    else:
      return ' | '.join([value[1],dn_str])
      

class Boolean(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.7'
  desc = 'Boolean'

  def validate(self,attrValue):
    
    return (attrValue=='TRUE') or (attrValue=='FALSE')

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return self.attrValue.encode('ascii')

  def formField(self):
    return pyweblib.forms.Select(
      self.attrType,
      ': '.join([self.attrType,self.desc]),1,
      options=('','TRUE','FALSE'),required=0,default=self.attrValue
    )


class CaseExactString(DirectoryString):
  oid = '1.3.6.1.4.1.1466.115.121.1.26'
  desc = 'Case Exact String'


class GeneralizedTime(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.24'
  desc = 'Generalized Time'
  reObj=timestamp_regex

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    try:
      return utctime.strftimeiso8601(utctime.strptime(self.attrValue)).encode('ascii')
    except ValueError:
      return utf2display(form.accept_charset,self.attrValue)


class UTCTime(GeneralizedTime):
  oid = '1.3.6.1.4.1.1466.115.121.1.53'
  desc = 'UTC Time'


class IA5String(DirectoryString):
  oid = '1.3.6.1.4.1.1466.115.121.1.26'
  desc = 'IA5 String'
  charset = 'ascii'


class Integer(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.27'
  desc = 'Integer'

  def ldapEncoding(self):
    return str(self.attrValue)

  def validate(self,attrValue):
    try:
      int(attrValue)
    except ValueError:
      return 0
    else:
      return 1

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return self.attrValue.encode('ascii')


class IntegerRange(Integer):
  oid = 'IntegerRange-oid'
  desc = 'Integer within a certain range'
  minValue = None
  maxValue = None

  def validate(self,attrValue):
    try:
      intValue = int(attrValue)
    except ValueError:
      return 0
    else:
      return (intValue>=self.minValue) and (intValue<=self.maxValue)


class RFC822Address(IA5String):
  oid = 'RFC822Address-oid'
  desc = 'RFC 822 mail address'
  reObj= re.compile(mail_pattern)

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return w2lapp.gui.MailToURL(self.attrValue)


class Uri(DirectoryString):
  oid = 'Uri-OID'
  desc = 'URI'
  reObj = url_regex

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    try:
      url,label = self.attrValue.split(' ',1)
    except ValueError:
      url,label = self.attrValue,self.attrValue
      display_url = ''
    else:
      display_url = ' (%s)' % (url)
    return '<a href="%s/urlredirect?%s">%s%s</a>' % (
      form.script_name,
      url.encode('ascii'),
      utf2display(form.accept_charset,label),
      utf2display(form.accept_charset,display_url),
    )


class JPEGImage(Binary):
  oid = '1.3.6.1.4.1.1466.115.121.1.28'
  desc = 'JPEG image'
  mimeType = 'image/jpeg'
  fileExt = 'jpg'

  def validate(self,attrValue):
    return imghdr.test_jpeg(attrValue,None)=='jpeg'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return """
      <img
        src="%s/read/%s?dn=%s&amp;read_attr=%s&amp;read_attrindex=%d&amp;read_attrmode=load&amp;read_attrmimetype=image/jpeg"
        alt="%d bytes of image data"
      >
      """ % (
      form.script_name,sid,
      urllib.quote(utf2display(form.accept_charset,dn,escape_entities=0)),
      urllib.quote(self.attrType),
      valueindex,len(self.attrValue),
    )
#    return """
#      <img
#        src="data:image/jpeg;base64,%s"
#        alt="%d bytes of image data"
#      >
#      """ % (base64.encodestring(self.attrValue),len(self.attrValue))


class PhotoG3Fax(Binary):
  oid = '1.3.6.1.4.1.1466.115.121.1.23'
  desc = 'Photo (G3 fax)'
  mimeType = 'image/tiff'
  fileExt = 'tif'


class OID(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.38'
  desc = 'OID'
  regex = '([0-9]+.)*[0-9]'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    attrValue = self.attrValue.encode('ascii')
    if oid_desc_reg.has_key(attrValue):
      name,description,reference = oid_desc_reg[self.attrValue]
      return '<strong>%s</strong> (%s):<br>%s (see %s)' % (
        name,attrValue,description,reference
      )
    else:
      return attrValue


class LDAPUrl(DirectoryString):
  oid = 'LDAPUrl-oid'
  desc = 'LDAP URL'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    try:
      if commandbutton:
        commandbuttonstr = w2lapp.gui.LDAPURLButton(sid,form,ls,self.attrValue)
      else:
        commandbuttonstr = ''
      return '<table><tr><td>%s</td><td><a href="%s">%s</a></td></tr></table>' % (
               commandbuttonstr,
	       self.attrValue.encode(form.accept_charset),
	       utf2display(form.accept_charset,self.attrValue)
	     )
    except ValueError:
      return '<strong>Not a valid LDAP URL:</strong> %s' % utf2display(form.accept_charset,repr(self.attrValue))


class OctetString(Binary):
  oid = '1.3.6.1.4.1.1466.115.121.1.40'
  desc = 'Octet String'
  editable = 1

  def sanitizeInput(self,attrValue):
    for c in [':',' ',',','\r','\n']:
      attrValue = attrValue.replace(c,'')
    result = []
    for i in range(0,len(attrValue),2):
      result.append(chr(int(attrValue[i:i+2],16)))
    result_str = ''.join(result)
    return result_str

  def validate(self,attrValue):
    return 1

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return '<code><font size="-1">%s</font></code>' % mspki.util.HexString(
      self.attrValue,
      delimiter=':',wrap=64,linesep='<br>'
    )

  def formField(self):
    attrValue_default = mspki.util.HexString(str(self.attrValue),delimiter='\r\n',wrap=40,linesep='')
    return pyweblib.forms.Textarea(
      self.attrType,
      ': '.join([self.attrType,self.desc]),
      10000,1,
      self.reObj.pattern,
      default=attrValue_default,
      rows=3,cols=41
    )


class PostalAddress(DirectoryString):
  oid = '1.3.6.1.4.1.1466.115.121.1.41'
  desc = 'Postal Address'
  reObj=re.compile('^.*$',re.S+re.M)
  def _split_lines(self,v):
    return v.split('$')

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    lines = [
      utf2display(form.accept_charset,l)
      for l in self._split_lines(self.attrValue)
    ]
    return '<br>'.join(lines)

  """
  def validate(self,attrValue):
    return self.reObj.match(self.attrValue)!=None

  def setValue(self,attrValue):
    attrValue = unicode(attrValue,self._charset)
    lines = attrValue.split('\n')
    for i in range(len(lines)):
      if lines[i].endswith('\r'):
        lines[i] = lines[i][:-1]
    self.attrValue = '$'.join(lines)

  def formField(self):
    return pyweblib.forms.Textarea(
      self.attrType,
      ': '.join([self.attrType,self.desc]),
      self.maxLen,self.maxValues,
      self.reObj.pattern,
      default=u'\r\n'.join(self._split_lines(self.attrValue)),
      rows=3,cols=40
    )
  """


class PrintableString(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.44'
  desc = 'Printable String'
  reObj= re.compile('^[a-zA-Z0-9"()+,.=/:? -]*$')

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return self.attrValue.encode('ascii')


class TelephoneNumber(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.50'
  desc = 'Telephone Number'
  reObj= re.compile('^\+[0-9 ]+$')

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return self.attrValue.encode('ascii')


class FacsimileTelephoneNumber(TelephoneNumber):
  oid = '1.3.6.1.4.1.1466.115.121.1.22'


class ObjectGUID(LDAPSyntax):
  oid = 'ObjectGUID-oid'
  desc = 'Object GUID'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    objectguid_str = ''.join(['%02X' % ord(c) for c in self.attrValue])
    return ldapurl.LDAPUrl(
      ldapUrl=ls.uri,
      dn='GUID=%s' % (objectguid_str),
      who=None,cred=None
    ).htmlHREF(
      hrefText=objectguid_str,
      hrefTarget=None
    )


class SecondsSinceEpoch(Integer):
  oid = 'SecondsSinceEpoch-oid'
  desc = 'Seconds since epoch (1970-01-01 00:00:00)'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    try:
      return utctime.strftimeiso8601(time.gmtime(float(self.attrValue))).encode('ascii')
    except ValueError:
      return utf2display(form.accept_charset,self.attrValue)


class DaysSinceEpoch(Integer):
  oid = 'DaysSinceEpoch-oid'
  desc = 'Days since epoch (1970-01-01)'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    try:
      return utctime.strftimeiso8601(time.gmtime(float(self.attrValue)*86400)).encode('ascii')
    except ValueError:
      return utf2display(form.accept_charset,self.attrValue)


class SelectList(LDAPSyntax):
  """
  Base class for dictionary based select lists which
  should not be used directly
  """

  attr_value_dict = {}
  
  def _sortedSelectOptions(self):
    tmplist = [(x[1], x) for x in self.attr_value_dict.items()]
    tmplist.sort()
    return [x for (k,x) in tmplist]
    
  def validate(self,attrValue):
    return self.attr_value_dict.has_key(attrValue)

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    attr_value_str = self.attrValue.encode('ascii')
    if self.attr_value_dict.has_key(attr_value_str):
      return ': '.join([attr_value_str,self.attr_value_dict[attr_value_str]])
    else:
      return attr_value_str

  def formField(self):
    return pyweblib.forms.Select(
      self.attrType,
      ': '.join([self.attrType,self.desc]),1,
      options=self._sortedSelectOptions(),required=0,default=self.attrValue
    )


class CountryString(SelectList):
  oid = '1.3.6.1.4.1.1466.115.121.1.11'
  desc = 'Two letter country string as listed in ISO 3166-2'

  attr_value_dict = ldap.cidict.cidict({
    "":"",
    "AF":"AFGHANISTAN",
    "AL":"ALBANIA",
    "DZ":"ALGERIA",
    "AS":"AMERICAN SAMOA",
    "AD":"ANDORRA",
    "AO":"ANGOLA",
    "AI":"ANGUILLA",
    "AQ":"ANTARCTICA",
    "AG":"ANTIGUA AND BARBUDA",
    "AR":"ARGENTINA",
    "AM":"ARMENIA",
    "AW":"ARUBA",
    "AU":"AUSTRALIA",
    "AT":"AUSTRIA",
    "AZ":"AZERBAIJAN",
    "BS":"BAHAMAS",
    "BH":"BAHRAIN",
    "BD":"BANGLADESH",
    "BB":"BARBADOS",
    "BY":"BELARUS",
    "BE":"BELGIUM",
    "BZ":"BELIZE",
    "BJ":"BENIN",
    "BM":"BERMUDA",
    "BT":"BHUTAN",
    "BO":"BOLIVIA",
    "BA":"BOSNIA AND HERZEGOVINA",
    "BW":"BOTSWANA",
    "BV":"BOUVET ISLAND",
    "BR":"BRAZIL",
    "IO":"BRITISH INDIAN OCEAN TERRITORY",
    "BN":"BRUNEI DARUSSALAM",
    "BG":"BULGARIA",
    "BF":"BURKINA FASO",
    "BI":"BURUNDI",
    "KH":"CAMBODIA",
    "CM":"CAMEROON",
    "CA":"CANADA",
    "CV":"CAPE VERDE",
    "KY":"CAYMAN ISLANDS",
    "CF":"CENTRAL AFRICAN REPUBLIC",
    "TD":"CHAD",
    "CL":"CHILE",
    "CN":"CHINA",
    "CX":"CHRISTMAS ISLAND",
    "CC":"COCOS (KEELING) ISLANDS",
    "CO":"COLOMBIA",
    "KM":"COMOROS",
    "CG":"CONGO",
    "CD":"CONGO, THE DEMOCRATIC REPUBLIC OF THE",
    "CK":"COOK ISLANDS",
    "CR":"COSTA RICA",
    "CI":"COTE D'IVOIRE",
    "HR":"CROATIA",
    "CU":"CUBA",
    "CY":"CYPRUS",
    "CZ":"CZECH REPUBLIC",
    "DK":"DENMARK",
    "DJ":"DJIBOUTI",
    "DM":"DOMINICA",
    "DO":"DOMINICAN REPUBLIC",
    "EC":"ECUADOR",
    "EG":"EGYPT",
    "SV":"EL SALVADOR",
    "GQ":"EQUATORIAL GUINEA",
    "ER":"ERITREA",
    "EE":"ESTONIA",
    "ET":"ETHIOPIA",
    "FK":"FALKLAND ISLANDS (MALVINAS)",
    "FO":"FAROE ISLANDS",
    "FJ":"FIJI",
    "FI":"FINLAND",
    "FR":"FRANCE",
    "GF":"FRENCH GUIANA",
    "PF":"FRENCH POLYNESIA",
    "TF":"FRENCH SOUTHERN TERRITORIES",
    "GA":"GABON",
    "GM":"GAMBIA",
    "GE":"GEORGIA",
    "DE":"GERMANY",
    "GH":"GHANA",
    "GI":"GIBRALTAR",
    "GR":"GREECE",
    "GL":"GREENLAND",
    "GD":"GRENADA",
    "GP":"GUADELOUPE",
    "GU":"GUAM",
    "GT":"GUATEMALA",
    "GN":"GUINEA",
    "GW":"GUINEA-BISSAU",
    "GY":"GUYANA",
    "HT":"HAITI",
    "HM":"HEARD ISLAND AND MCDONALD ISLANDS",
    "VA":"HOLY SEE (VATICAN CITY STATE)",
    "HN":"HONDURAS",
    "HK":"HONG KONG",
    "HU":"HUNGARY",
    "IS":"ICELAND",
    "IN":"INDIA",
    "ID":"INDONESIA",
    "IR":"IRAN, ISLAMIC REPUBLIC OF",
    "IQ":"IRAQ",
    "IE":"IRELAND",
    "IL":"ISRAEL",
    "IT":"ITALY",
    "JM":"JAMAICA",
    "JP":"JAPAN",
    "JO":"JORDAN",
    "KZ":"KAZAKHSTAN",
    "KE":"KENYA",
    "KI":"KIRIBATI",
    "KP":"KOREA, DEMOCRATIC PEOPLE'S REPUBLIC OF",
    "KR":"KOREA, REPUBLIC OF",
    "KW":"KUWAIT",
    "KG":"KYRGYZSTAN",
    "LA":"LAO PEOPLE'S DEMOCRATIC REPUBLIC",
    "LV":"LATVIA",
    "LB":"LEBANON",
    "LS":"LESOTHO",
    "LR":"LIBERIA",
    "LY":"LIBYAN ARAB JAMAHIRIYA",
    "LI":"LIECHTENSTEIN",
    "LT":"LITHUANIA",
    "LU":"LUXEMBOURG",
    "MO":"MACAO",
    "MK":"MACEDONIA, THE FORMER YUGOSLAV REPUBLIC OF",
    "MG":"MADAGASCAR",
    "MW":"MALAWI",
    "MY":"MALAYSIA",
    "MV":"MALDIVES",
    "ML":"MALI",
    "MT":"MALTA",
    "MH":"MARSHALL ISLANDS",
    "MQ":"MARTINIQUE",
    "MR":"MAURITANIA",
    "MU":"MAURITIUS",
    "YT":"MAYOTTE",
    "MX":"MEXICO",
    "FM":"MICRONESIA, FEDERATED STATES OF",
    "MD":"MOLDOVA, REPUBLIC OF",
    "MC":"MONACO",
    "MN":"MONGOLIA",
    "MS":"MONTSERRAT",
    "MA":"MOROCCO",
    "MZ":"MOZAMBIQUE",
    "MM":"MYANMAR",
    "NA":"NAMIBIA",
    "NR":"NAURU",
    "NP":"NEPAL",
    "NL":"NETHERLANDS",
    "AN":"NETHERLANDS ANTILLES",
    "NC":"NEW CALEDONIA",
    "NZ":"NEW ZEALAND",
    "NI":"NICARAGUA",
    "NE":"NIGER",
    "NG":"NIGERIA",
    "NU":"NIUE",
    "NF":"NORFOLK ISLAND",
    "MP":"NORTHERN MARIANA ISLANDS",
    "NO":"NORWAY",
    "OM":"OMAN",
    "PK":"PAKISTAN",
    "PW":"PALAU",
    "PS":"PALESTINIAN TERRITORY, OCCUPIED",
    "PA":"PANAMA",
    "PG":"PAPUA NEW GUINEA",
    "PY":"PARAGUAY",
    "PE":"PERU",
    "PH":"PHILIPPINES",
    "PN":"PITCAIRN",
    "PL":"POLAND",
    "PT":"PORTUGAL",
    "PR":"PUERTO RICO",
    "QA":"QATAR",
    "RE":"REUNION",
    "RO":"ROMANIA",
    "RU":"RUSSIAN FEDERATION",
    "RW":"RWANDA",
    "SH":"SAINT HELENA",
    "KN":"SAINT KITTS AND NEVIS",
    "LC":"SAINT LUCIA",
    "PM":"SAINT PIERRE AND MIQUELON",
    "VC":"SAINT VINCENT AND THE GRENADINES",
    "WS":"SAMOA",
    "SM":"SAN MARINO",
    "ST":"SAO TOME AND PRINCIPE",
    "SA":"SAUDI ARABIA",
    "SN":"SENEGAL",
    "SC":"SEYCHELLES",
    "SL":"SIERRA LEONE",
    "SG":"SINGAPORE",
    "SK":"SLOVAKIA",
    "SI":"SLOVENIA",
    "SB":"SOLOMON ISLANDS",
    "SO":"SOMALIA",
    "ZA":"SOUTH AFRICA",
    "GS":"SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS",
    "ES":"SPAIN",
    "LK":"SRI LANKA",
    "SD":"SUDAN",
    "SR":"SURINAME",
    "SJ":"SVALBARD AND JAN MAYEN",
    "SZ":"SWAZILAND",
    "SE":"SWEDEN",
    "CH":"SWITZERLAND",
    "SY":"SYRIAN ARAB REPUBLIC",
    "TW":"TAIWAN, PROVINCE OF CHINA",
    "TJ":"TAJIKISTAN",
    "TZ":"TANZANIA, UNITED REPUBLIC OF",
    "TH":"THAILAND",
    "TL":"TIMOR-LESTE",
    "TG":"TOGO",
    "TK":"TOKELAU",
    "TO":"TONGA",
    "TT":"TRINIDAD AND TOBAGO",
    "TN":"TUNISIA",
    "TR":"TURKEY",
    "TM":"TURKMENISTAN",
    "TC":"TURKS AND CAICOS ISLANDS",
    "TV":"TUVALU",
    "UG":"UGANDA",
    "UA":"UKRAINE",
    "AE":"UNITED ARAB EMIRATES",
    "GB":"UNITED KINGDOM",
    "US":"UNITED STATES",
    "UM":"UNITED STATES MINOR OUTLYING ISLANDS",
    "UY":"URUGUAY",
    "UZ":"UZBEKISTAN",
    "VU":"VANUATU",
    "VE":"VENEZUELA",
    "VN":"VIET NAM",
    "VG":"VIRGIN ISLANDS, BRITISH",
    "VI":"VIRGIN ISLANDS, U.S.",
    "WF":"WALLIS AND FUTUNA",
    "EH":"WESTERN SAHARA",
    "YE":"YEMEN",
    "YU":"YUGOSLAVIA",
    "ZM":"ZAMBIA",
    "ZW":"ZIMBABWE",
  })


# Set up the central syntax registry instance
syntax_registry = SyntaxRegistry()
# Register all syntax classes in this module
for symbol_name in dir():
  syntax_registry.registerSyntaxClass(eval(symbol_name))

syntax_registry.registerAttrType(
  OID.oid,
  [
    '1.2.826.0.1050.11.0','ogSupportedProfile',
    '1.3.6.1.4.1.1466.101.120.13','supportedControl',
    '1.3.6.1.4.1.1466.101.120.7','supportedExtension',
    '1.3.6.1.4.1.4203.1.3.5','supportedFeatures',
  ]
)

syntax_registry.registerAttrType(
  RFC822Address.oid,
  [
    '0.9.2342.19200300.100.1.3',    # mail, rfc822Mailbox
    '2.16.840.1.113730.3.1.13',     # mailLocalAddress
    '2.16.840.1.113730.3.1.17',     # mailForwardingAddress
    '2.16.840.1.113730.3.1.30',     # mgrpRFC822MailMember
    '2.16.840.1.113730.3.1.47',     # mailRoutingAddress
    '1.2.840.113549.1.9.1',         # email, emailAddress, pkcs9email
    '2.16.840.1.113678.2.2.2.2.15', # Domino/LDAP: MailAddress
  ]
)

syntax_registry.registerAttrType(
  JPEGImage.oid,
  [
    '0.9.2342.19200300.100.1.60', # jpegPhoto
  ]
)

syntax_registry.registerAttrType(
  Audio.oid,
  [
    '0.9.2342.19200300.100.1.55', # audio
  ]
)

syntax_registry.registerAttrType(
  Uri.oid,[
    '1.3.6.1.4.1.250.1.57', # labeledURI
  ]
)

syntax_registry.registerAttrType(
  Boolean.oid,[
    '2.5.18.9', # hasSubordinates
  ]
)

syntax_registry.registerAttrType(
  PostalAddress.oid,[
    '2.5.4.16',                   # postalAddress
    '2.5.4.26',                   # registeredAddress
    '0.9.2342.19200300.100.1.39', # homePostalAddress
  ]
)

syntax_registry.registerAttrType(
  LDAPUrl.oid,[
    '2.16.840.1.113730.3.1.34', # ref
  ]
)

syntax_registry.registerAttrType(
  Certificate.oid,[
    '2.5.4.36', # userCertificate
    '2.5.4.37', # cACertificate
    'userCertificate','userCertificate;binary',
    'cACertificate','cACertificate;binary',
  ]
)

syntax_registry.registerAttrType(
  CertificateRevocationList.oid,[
    '2.5.4.38', # authorityRevocationList
    '2.5.4.39', # certificateRevocationList
    '2.5.4.53', # deltaRevocationList
    'authorityRevocationList','authorityRevocationList;binary',
    'certificateRevocationList','certificateRevocationList;binary',
    'deltaRevocationList','deltaRevocationList;binary',
  ]
)

syntax_registry.registerAttrType(
  UTCTime.oid,[
    '2.5.18.1', # createTimestamp
    '2.5.18.2', # modifyTimestamp
    'createtimestamp-oid', # createtimestamp on Netscape DS 4.x
    'modifytimestamp-oid', # modifytimestamp on Netscape DS 4.x
  ]
)

syntax_registry.registerAttrType(
  CountryString.oid,[
    'c',
    'countryName',
    '2.5.4.6', # c
  ]
)
